# /**
#  * @file  Statistic.pm
#  * @brief Computing some statistical values 
#  *
#  * @author M.Sekido (NICT Kashima.)
#  * @date $Date: 2017-04-22 16:56:24 +0900 (Sat, 22 Apr 2017) $
#  * @version $Id: Statistic.pm 121 2017-04-22 07:56:24Z vlbi $
#  * 
#  * Copyright (C) 2009 NICT Kashima. All rights reserved.
#  */

package Statistic;

use Exporter;
@ISA = qw(Exporter);

@EXPORT = qw(Compute);

BEGIN {
#
}

sub new ($){
    my $pkg = shift;
    my $this = {
    };
    bless $this;
    $this;
}
## Compute avg, sigma, max, min, <(Xi+1 - Xi)^2>
## @param arr : data array
## @param ncmp : Number of point used for computation
## @param excld: If specified, data outside from sigma*excld is exluded.
sub Compute($$;$$){
    my $pkg = shift;
    my $arr = shift;
    my $ncmp = shift;  # Number of points used for computing
    my $excld = shift;
    unless(defined($arr)){
	print STDERR "E Statisitc::Compute(): data array is not defined.\n";
	return -1;
    }
    my $num = @$arr;
    $num = $ncmp if(defined($ncmp)) ; # If $ncmp is specified, 
    if( $num ==0){
	print STDERR "Error in Statistic::Compute(): Ndata=0\n";
	return -1;
    }
    my $mx =0;
    my $max = $$arr[0];
    my $min = $$arr[0];
    my $cnt=0;
    for(my $i=0; $i<$num; $i++){ 
	next unless(defined($$arr[$i]));
	$mx += $$arr[$i];   
	$max = ($max < $$arr[$i]) ? $$arr[$i] : $max;
	$min = ($min > $$arr[$i]) ? $$arr[$i] : $min;
	$cnt++;
    }
    my $avg = $mx/$cnt;

    $mx =0;
    $cnt=0;
    for(my $i=0; $i<$num; $i++){ 
	next unless(defined($$arr[$i]));
	my $x = $$arr[$i] - $avg;
	$mx +=  $x * $x;
	$cnt++;
    }
    my $sig = sqrt($mx/($cnt -1.));

    #------  <(Xi+1 - Xi)^2> ---
    $cnt=0;
    $mx=0;
    for(my $i=0; $i<$num-1; $i++){ 
	next unless(defined($$arr[$i]) && defined($$arr[$i+1]));
	my $x = $$arr[$i+1] - $$arr[$i];
	$mx +=  $x * $x;
	$cnt++;
    }
    my  $difsig = sqrt($mx/$cnt);

    if(defined($excld)){
	my $crt = $sig * $excld;
	my $cnt=0;
	$mx=0;
	for(my $i=0; $i<$num; $i++){
	    next if( abs($$arr[$i] - $avg) > $crt);
	    $mx += $$arr[$i];   
	    $cnt++;
	}
	$avg = $mx/$cnt;
	$pkg->{avg} = $avg;

	$mx =0;
	$cnt=0;
	for(my $i=0; $i<$num; $i++){ 
	    next if( abs($$arr[$i] - $avg) > $crt);
	    my $x = $$arr[$i] - $avg;
	    $mx +=  $x * $x;
	    $cnt++;
	}
	$sig = sqrt($mx/($cnt-1.));
	
	#------  <(Xi+1 - Xi)^2> ---
	$cnt=0;
	$mx=0;
	for(my $i=0; $i<$num-1; $i++){ 
	    next if( abs($$arr[$i] - $avg) > $crt);
	    my $x = $$arr[$i+1] - $$arr[$i];
	    $mx +=  $x * $x;
	    $cnt++;
	}
	$difsig = sqrt($mx/$cnt);
	
    }

    $pkg->{sig} = $sig;
    $pkg->{avg} = $avg;
    $pkg->{max} = $max;
    $pkg->{min} = $min;
    $pkg->{npt} = $cnt;
    $pkg->{difsig} = $difsig;

    return 0;
}




# Computing Avg and Sigm from averaged data
sub Compute_Navg($$$$;$){
    my $pkg = shift;
    my $datarr = shift;
    my $sigarr = shift;
    my $nptarr = shift;
    my $ncmp = shift;  # Number of points used for computing(average)
    unless(defined($datarr) && defined($sigarr) && defined($nptarr)){
	print STDERR "E Statisitc::Compute_wSig(): data array isn't defined.\n";
	return -1;
    }
    my $ndat = @$datarr;
    $ncmp =$ndat unless(defined($ncmp)) ; # If $ncmp is specified, 
    my $max = $$datarr[0];
    my $min = $$datarr[0];
    my (@newavg, @newsig, @newnpt);
    my $nc=0;
    my $icnt=0;
    while($nc +$ncmp <= $ndat){
	my $avg=$$datarr[$nc];
	my $sum_n = $$nptarr[$nc];
	my $mx2 = $$sigarr[$nc] * $$sigarr[$nc] 
	    + $$datarr[$nc] * $$datarr[$nc];
	for(my $i=1; $i<$ncmp; $i++){ 
	    my $sum_n1 = $sum_n + $$nptarr[$i+$nc];
	    my $fact1 = $sum_n/$sum_n1;
	    my $fact2 = $$nptarr[$i+$nc]/$sum_n1;
	    $avg = $fact1 * $avg  + $fact2* $$datarr[$i+$nc];
	    my $s2 =  $$sigarr[$i+$nc] * $$sigarr[$i+$nc]
		+ $$datarr[$i+$nc] * $$datarr[$i+$nc];
	    $mx2 = $fact1 * $mx2 + $fact2 * $s2;
	    $sum_n = $sum_n1;
	    $max = ($max < $$datarr[$i+$nc]) ? $$datarr[$i+$nc] : $max;
	    $min = ($min > $$datarr[$i+$nc]) ? $$datarr[$i+$nc] : $min;
	}
	$newavg[$icnt] = $avg;
	$newsig[$icnt] = sqrt($mx2-$avg*$avg);
	$newnpt[$icnt] = $sum_n;
	$icnt++;
	$nc += $ncmp;
    }
    $pkg->{avgarr} = \@newavg;
    $pkg->{sigarr} = \@newsig;
    $pkg->{nptarr} = \@newnpt;
    $pkg->Compute(\@newavg) if(@newavg >2);
    $pkg->{max} = $max;
    $pkg->{min} = $min;

    return $icnt;
}

sub Avgarr($){
    my $pkg = shift;
    return   $pkg->{avgarr};
}
sub Sigarr($){
    my $pkg = shift;
    return   $pkg->{sigarr};
}
sub Nptarr($){
    my $pkg = shift;
    return   $pkg->{nptarr};
}

sub Avg($){
    my $pkg = shift;
    return   $pkg->{avg};
}

sub Sig($){
    my $pkg = shift;
    return   $pkg->{sig};
}

sub Max($){
    my $pkg = shift;
    return   $pkg->{max};
}

sub Min($){
    my $pkg = shift;
    return   $pkg->{min};
}

sub Npt($){
    my $pkg = shift;
    return   $pkg->{npt};
}

## sqrt(<(Xi+1 - Xi)^2>)
sub DifSig($){
    my $pkg = shift;
    return   $pkg->{difsig};
}

sub Navg($$$){
    my $pkg = shift;
    my $arr = shift;
    my $navg = shift;
    
    my $npt = @$arr;
    my $nc =0;
    my @narr;
    my $cnt=0;
    while( $nc + $navg < $npt){
	my $mx =0;
	for(my $i = $nc; $i<$nc + $navg; $i++){ $mx += $$arr[$i];}
	$narr[$cnt++] = $mx/$navg;
	$nc += $navg;
    }
    $pkg->Compute(\@narr) if(@narr > 2);
    return @narr;
}

1;
